//===- AnalysisPass.h - AutoJIT Analysis Pass -------------------*- C++ -*-===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//
//===----------------------------------------------------------------------===//

#ifndef AUTOJIT_ANALYSISPASS_H
#define AUTOJIT_ANALYSISPASS_H

#include <set>
#include <map>
#include <vector>
#include <list>

#include "llvm/Pass.h"

namespace llvm {
  class Loop;
  class BasicBlock;
  class Instruction;
  class CallInst;
  class LoadInst;
  class PHINode;
  class Value;
}

namespace autojit {
  class AnalysisPass : public llvm::FunctionPass {
public:
    enum FwdStatus {
      Invariant,
      InstPtr,
      InstData,
      Excluded,
    };

    static char ID;
    AnalysisPass() : FunctionPass(&ID) {}

    const std::set<llvm::CallInst*>& getAnnotations() const;
    llvm::BasicBlock* getHead() const;
    const std::set<llvm::BasicBlock*>& getBodySet() const;
    const std::list<llvm::BasicBlock*>& getBodyList() const;
    const std::set<llvm::Instruction*>& getControlFlowValues() const;
    const std::set<llvm::Loop*>& getControlFlowLoops() const;
    const std::set<llvm::Value*>& getInvariants() const;
    const std::set<llvm::PHINode*>& getControlVariants() const;
    const std::set<llvm::PHINode*>& getRuntimeVariants() const;
    const std::set<llvm::Instruction*>& getSpecialiseValues() const;

    virtual bool runOnFunction(llvm::Function&);
    virtual void getAnalysisUsage(llvm::AnalysisUsage&) const;

private:
    std::set<llvm::CallInst*> mAnnotations;
    std::set<llvm::LoadInst*> mInstLoads;
    llvm::BasicBlock* mHead;
    std::set<llvm::Value*> mInvars;
    std::set<llvm::PHINode*> mCtrlVars;
    std::set<llvm::PHINode*> mRtVars;
    std::set<llvm::BasicBlock*> mBody;
    std::list<llvm::BasicBlock*> mBodyList;
    std::set<llvm::Instruction*> mSpecVals;
    std::set<llvm::Loop*> mCtrlFlowLoops;

    typedef std::map<llvm::Instruction*, FwdStatus> FwdScan;
    typedef std::pair<llvm::Instruction*, FwdStatus> FwdPair;
    typedef std::set<llvm::Instruction*> BwdScan;
    BwdScan mCtrlFlowVals;
  };
}

#endif /*AUTOJIT_ANALYSISPASS_H*/
