//===- BranchMerge.h - Branch/Merge Correspondence --------------*- C++ -*-===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//
//===----------------------------------------------------------------------===//

#ifndef AUTOJIT_BRANCHMERGE_H
#define AUTOJIT_BRANCHMERGE_H

#include <set>
#include <map>
#include <vector>

#include "llvm/Pass.h"
#include "llvm/Instructions.h"
#include "llvm/BasicBlock.h"
#include "llvm/Function.h"

namespace autojit {
  class BranchMerge : public llvm::FunctionPass {
public:
    static char ID;
    BranchMerge() : FunctionPass(&ID) {}

    typedef std::set<llvm::TerminatorInst*> BranchSet;
    typedef std::set<llvm::BasicBlock*> MergeSet;

    const BranchSet& getBranchesForMerge(llvm::BasicBlock*);
    const MergeSet& getMergesForBranch(llvm::TerminatorInst*);
    const MergeSet& getUnmerged();

    virtual bool runOnFunction(llvm::Function&);
    virtual void getAnalysisUsage(llvm::AnalysisUsage&) const;

private:
    typedef std::map<llvm::BasicBlock*, BranchSet> BranchSets;
    typedef std::map<llvm::TerminatorInst*, MergeSet> MergeSets;

    BranchSets mBranchSets;
    MergeSets mMergeSets;
    MergeSet mUnmerged;
  };
}

#endif /*AUTOJIT_BRANCHMERGE_H*/
